
      program clebsch_test
	 use dflib            ! here GETARG is taken from
	common/idents/idcomm,NProc,myProc 
	parameter (nrep=100)  ! number of repears
      data eps/1.0E-50/     ! accuracy for comparison
	character*80 ComLine/''/   ! parameter string
	real*8 clebsch2
      CALL GETARG(1,COMLINE)     ! parameter passed into MPI-executable 
      comline='2'

      idcomm=0                       ! not initializsed yet  
	c=clebsch(0,0)                 ! to initialise and get myProc&Nproc
      if(myProc.eq.0) write(*,*) 'Clebsch test ',Trim(Comline),
     -  ' repeated',nrep,' times'
	call profile_start             ! start timing
	mer=0
      do NN=1,nrep
	  N=10000  ! nn
	  do K=1,N,NProc ! to perform equal job for each process
	   if(comline.eq.'') then
	    c1=Clebsch(K,N)
	    c2=Clebsch(N-K,N)
         else
	    c1=Clebsch2(K,N)
	    c2=Clebsch2(N-K,N)
	   endif
          if((c1.gt.1.).or.(c2.gt.1.).or.(abs(c1-c2).gt.eps)) then
	      write(*,*) ' Incorrect result:',K,N,c1,c2
	      mer=mer+1
          endif
	  enddo
	enddo
	call profile_fin             ! end timing
	if(myProc+mer.eq.0) write(*,*) ' Correct result!'
!   It is interesting: which Clebsch will be faster ?
          call cpu_time(t1)
          do i=1,10000
	      c1=Clebsch(i/2,i)
	    enddo
          call cpu_time(t2)
	    if(myProc.eq.0) write(*,*) ' 1 : ', t2-t1
          call cpu_time(t1)
          do i=1,10000
	      c2=Clebsch2(i/2,i)
	    enddo
          call cpu_time(t2)
	    if(myProc.eq.0) write(*,*) ' 2 : ', t2-t1
!   check results for both versions:
	    do i=1,10000  
	      c1=Clebsch(i/2,i)
	      c2=Clebsch2(i/2,i)
	      if(abs(c1-c2).gt.1.0E-8) write(*,*) i,c1,c2,c1-c2
	    enddo
      end
      
	function Clebsch(k,n)  ! calc Clebsch-Gordan simplifying coef = K! * (n-k)! / n!
      Include 'mpif.h'
	common/idents/idcomm,NProc,myProc  
	real*8 summ,c1,c2,c3,alog 
	if (idcomm.eq.0) then  ! MPI Initialisation
	  idcomm=MPI_Comm_World 
        call MPI_Init(ierr)                         ! Standard start of any MPI-program
        call MPI_Comm_Size(idcomm, NProc, ierr)     ! How many processes are there ?
        call MPI_Comm_Rank(idcomm, myProc, ierr)    ! Who am I ?
	endif
	if((n.lt.0).or.(k.lt.0).or.(k.gt.n)) then
	  if (myProc.eq.0) write(*,*) ' Clebsch: illegal parameters! ',k,n
       pause
	  stop
	endif
      c1=0.0d0
      c2=0.0d0
      c3=0.0d0
      do i=1+myProc,N,NProc      ! each process calculates only its own factorials !!! 
	  Alog=DLog(DBLE(i))       ! We use logarithms to avoid overflow!
        if(i.le.k) c1=c1+Alog    !  k!
        if(i.le.n-k) c2=c2+Alog  !  (n-k)!
        c3=c3-Alog               !  n!
      enddo
      call MPI_AllReduce(c1+c2+c3,summ,1,MPI_Double_Precision,
     -	 MPI_SUM, idcomm,ierr)
      Clebsch=DExp(summ)          ! Join all partial results
      return
      end

! Can I calc a bit faster?  I'll try (see below)

	Real*8 function Clebsch2(k,n)  ! calc Clebsch-Gordan simplifying coef = K! * (n-k)! / n!
	implicit real*8 (a-h,o-z)
      Include 'mpif.h'
	common/idents/idcomm,NProc,myProc 
	if (idcomm.eq.0) then  ! MPI Initialisation
	  idcomm=MPI_Comm_World 
        call MPI_Init(ierr)                         ! Standard start of any MPI-program
        call MPI_Comm_Size(idcomm, NProc, ierr)     ! How many processes are there ?
        call MPI_Comm_Rank(idcomm, myProc, ierr)    ! Who am I ?
	endif
	if((n.lt.0).or.(k.lt.0).or.(k.gt.n)) then
	  if (myProc.eq.0) write(*,*) ' Clebsch: illegal parameters! ',k,n
	  stop
	endif
      c=1.0
 	n1=min(k,N-k)  ! slightly reduced loop!
      do i=1+myProc,N1,NProc      ! each process calculates only its own factorials !!! 
       c=c*dble(i)/dble(N-i+1)
      enddo
	prod=c
      call MPI_AllReduce(c,prod,1,MPI_Double_Precision,
     -          MPI_PROD, idcomm,ierr)
      Clebsch2=prod          ! Join all partial results
      return
      end

